# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class BoardsController < ApplicationController
  layout 'base_projects'#by young
  default_search_scope :messages
  before_filter :find_project_by_project_id, :find_board_if_available
  before_filter :authorize, :except => [:new, :show, :create, :index]
  accept_rss_auth :index, :show
  

  helper :sort
  include SortHelper
  helper :watchers
  helper :project_score
  helper :attachments
  def index
    #modify by nwb
    @flag = params[:flag] || false
    if @project
      if !@project.is_public? && !User.current.member_of?(@project) && !User.current.admin?
        render_403
      else
        @boards = @project.boards.includes(:last_message => :author).all
        @boards = [] << @boards[0] if @boards.any?
        if @boards.size == 1
          @board = @boards.first
          show and return
        end
        render :layout => false if request.xhr?
      end
    elsif @course
      if (User.current.admin? || @course.is_public == 1 || (@course.is_public == 0 && User.current.member_of_course?(@course)))
        @boards = @course.boards.includes(:last_message => :author).all
        if @course.boards.empty?
          @board = @course.boards.build
          @board.name = " #{l(:label_borad_course) }"
          @board.description = @course.name.to_s
          @board.project_id = -1
          if @board.save
            @boards = @course.boards.includes(:last_message => :author).all
          end
        end
        unless @course.boards.empty?
          @board = @course.boards.first
        end
        show and return
      else
        render_403
      end

    end

  end

  def show
    # 讨论区消息状态更新（已读和未读）
    if @project
      query_forge_messages = @board.messages
      query_forge_messages.each do |query_forge_message|
        query = query_forge_message.forge_messages
        query.each do |forge_message|
          if User.current.id == forge_message.user_id
            forge_message.update_attributes(:viewed => true)
          end
        end
      end
    elsif @course
      query_course_messages = @board.messages
      query_course_messages.each do |query_course_message|
        query = query_course_message.course_messages
        query.each do |course_message|
          if User.current.id == course_message.user_id
            course_message.update_attributes(:viewed => true)
          end
        end
      end
    end

    sort_init 'updated_on', 'desc'
    sort_update 'created_on' => "#{Message.table_name}.created_on",
                'replies' => "#{Message.table_name}.replies_count",
                'updated_on' => "COALESCE(last_replies_messages.created_on, #{Message.table_name}.created_on)"

    @is_new = params[:is_new]
    @topic_count = @board ? @board.topics.count : 0
    if @project
      @topic_pages = Paginator.new @topic_count, per_page_option, params['page']
      #现在发布帖子的时候置顶功能已经没有了。所以取消这个置顶排序 #{Message.table_name}.sticky DESC,
      @topics = @board.topics.
          reorder("#{Message.table_name}.created_on desc").
          includes(:last_reply).
          limit(@topic_pages.per_page).
          offset(@topic_pages.offset).

          preload(:author, {:last_reply => :author}).
          all
    elsif @course
      #
      # board_topics =  @board ? @board.topics.reorder("#{Message.table_name}.sticky DESC, #{Message.table_name}.created_on desc").
      #     includes(:last_reply).
      #     # limit(@topic_pages.per_page).
      #     # offset(@topic_pages.offset).
      #
      #     preload(:author, {:last_reply => :author}).
      #     all : []
      # @topics = paginateHelper board_topics,10
      if (@board)
        limit = 10;
        #pageno = params[:page];
        #if(pageno == nil || pageno=='')
        #   dw_topic = nil;
        #  if( params[:parent_id]!=nil && params[:parent_id]!='' )
        #    dw_topic = @board.topics.where(id:params[:parent_id]).first();
        #  end
        #  if( dw_topic != nil )
        #    dw_count = @board.topics.where('(sticky>?) or (sticky=? and created_on>?)',dw_topic.sticky,dw_topic.sticky,dw_topic.created_on).count();
        #    dw_count = dw_count+1;
        #    pageno = dw_count%10==0 ? (dw_count/limit) : (dw_count/limit+1)
        #  end
        #end
        #if(pageno == nil || pageno=='')
        #  pageno=1;
        #end
        @topic_count = @board.topics.count();
        @topic_pages = (params[:page] ? params[:page].to_i + 1 : 0) *10
        @topics = @board.topics.reorder("#{Message.table_name}.sticky DESC, COALESCE(last_replies_messages.created_on, #{Message.table_name}.created_on) desc").
            limit(limit).offset(@topic_pages).includes(:last_reply).
            preload(:author, {:last_reply => :author}).all();
      else
        @topics = [];
      end
    end

    @message = Message.new(:board => @board)
    #modify by nwb
    respond_to do |format|
      format.js
      format.html {
        if @project
          render :action => 'show', :layout => 'base_projects'
        elsif @course
          @params=params
          render :action => 'show', :layout => 'base_courses'
        end
      }
      format.atom {
        @messages = @board.messages.
            reorder('created_on DESC').
            includes(:author, :board).
            limit(Setting.feeds_limit.to_i).
            all
        if @project
          render_feed(@messages, :title => "#{@project}: #{@board}")
        elsif @course
          render_feed(@messages, :title => "#{@course}: #{@board}")
        end

      }
    end
  end

  def new
    @board = @project.boards.build
    @board.safe_attributes = params[:board]
    if @project.project_type == 1
      render :layout => 'base_courses'
    end
  end

  def create    
    @board = @project.boards.build
    @board.safe_attributes = params[:board]
    
    if @board.save
      flash[:notice] = l(:notice_successful_create)
      #Modified by young
      #redirect_to_settings_in_projects
      redirect_to project_board_url(@project, @board)
      #Ended by young
    else
      render :action => 'new'
    end
  end

  def edit
    if @project.project_type == 1
      render :layout => 'base_courses'
    end
  end

  def update
    @board.safe_attributes = params[:board]
    if @board.save
      redirect_to_settings_in_projects
    else
      render :action => 'edit'
    end
  end

  def destroy
    @board.destroy
    redirect_to_settings_in_projects
  end

private
  def redirect_to_settings_in_projects
    redirect_to settings_project_url(@project, :tab => 'boards')
  end

  def find_board_if_available
    if @project
      @board = @project.boards.find(params[:id]) if params[:id]
    elsif @course
      @board = @course.boards.find(params[:id]) if params[:id]
    end
  rescue ActiveRecord::RecordNotFound
    render_404
  end
end
