# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class NewsController < ApplicationController
  layout 'base_projects'# by young
  include ApplicationHelper
  before_filter :authorize1, :only => [:show]
  default_search_scope :news
  model_object News
  before_filter :find_model_object, :except => [:new, :create, :index]
  before_filter :find_project_from_association, :except => [:new, :create, :index]
  before_filter :find_project_by_project_id, :only => [:new, :create]
  before_filter :authorize, :except => [:index]
  before_filter :find_optional_project, :only => :index
  accept_rss_auth :index
  accept_api_auth :index

  helper :watchers
  helper :attachments
  helper :project_score

  def index
    case params[:format]
    when 'xml', 'json'
      @offset, @limit = api_offset_and_limit
    else
      @limit =  10
    end

    # modify by nwb
    if params[:course_id] && @course==nil
      @course = Course.find(params[:course_id])
    end
    if @project
      scope = @project ? @project.news.visible : News.visible

      @news_count = scope.count
      @news_pages = Paginator.new @news_count, @limit, params['page']
      @offset ||= @news_pages.offset
      @newss = scope.all(:include => [:author, :project],
                         :order => "#{News.table_name}.created_on DESC",
                         :offset => @offset,
                         :limit => @limit)

      respond_to do |format|
        format.html {
          @news = News.new # for adding news inline
                           # huang

          render :layout => false if request.xhr?
        }
        format.api
        format.atom { render_feed(@newss, :title => (@project ? @project.name : Setting.app_title) + ": #{l(:label_news_plural)}") }
      end
    elsif @course
      if (User.current.admin? || @course.is_public == 1 || (@course.is_public == 0 && User.current.member_of_course?(@course)))
        scope = @course ? @course.news.course_visible : News.course_visible

        @news_count = scope.count
        @q = params[:subject]
        if params[:subject].nil? || params[:subject].blank?
          scope_order = scope.all(:include => [:author, :course],
                                  :order => "#{News.table_name}.sticky DESC, #{News.table_name}.created_on DESC")
        else
          scope_order = scope.where("#{News.table_name}.title like '#{'%' << params[:subject].to_s << '%'}'").all(:include => [:author, :course],
                                                                                    :order => "#{News.table_name}.sticky DESC, #{News.table_name}.created_on DESC")
        end

                           # :offset => @offset,
                           # :limit => @limit)
        @newss = paginateHelper scope_order,10
        respond_to do |format|
          format.html {
            @news = News.new
            render :layout => 'base_courses'
          }
          format.js
          format.api
          format.atom { render_feed(@newss, :title => (@course ? @course.name : Setting.app_title) + ": #{l(:label_news_plural)}") }
        end
      else
        render_403
      end
    end
   end

  def show
    # 更新news对应的forge_messages的消息viewed字段
    if @project
      query_message_news = @news.forge_messages
    else
      query_message_news = @news.course_messages
    end
    query_message_news.each do |query|
        if User.current.id == query.user_id
          query.update_attributes(:viewed => true)
        end
    end
    # 更新项目新闻的评阅的消息viewed字段
    current_message_comments = @news.comments
    current_message_comments.each do |current_message_comment|
    if @project
      query_message_comment = current_message_comment.forge_messages
    else
      query_message_comment = current_message_comment.course_messages
    end
    query_message_comment.each do |query|
        if User.current.id == query.user_id
          query.update_attributes(:viewed => true)
        end
      end
    end
    # end

    cs = CoursesService.new
    result = cs.show_course_news params,User.current
    @news = result[:news]
    @comments =  result[:comments]
    #@comments = @news.comments
    #@comments.reverse! if User.current.wants_comments_in_reverse_order?
    #modify by nwb
    if @news.course_id
      @course = Course.find(@news.course_id)
      if @course
        render :layout => 'base_courses'
      end
    end
  end

  def new
    #modify by nwb
    if @project
      @news = News.new(:project => @project, :author => User.current)
    elsif @course
      @news = News.new(:course => @course, :author => User.current)
      render :layout => 'base_courses'
    end
  end

  def create
    #modify by nwb
    if @project
      @news = News.new(:project => @project, :author => User.current)
      @news.safe_attributes = params[:news]
      @news.save_attachments(params[:attachments])
      if @news.save
        render_attachment_warning_if_needed(@news)
        flash[:notice] = l(:notice_successful_create)
        redirect_to project_news_index_url(@project)
      else
        layout_file = @project ? 'base_projects' : 'base_courses'
        render :action => 'new', :layout => layout_file
      end
    elsif @course
      @news = News.new(:course => @course, :author => User.current)
      @news.safe_attributes = params[:news]
      @news.save_attachments(params[:attachments])
      if @news.save
        if params[:asset_id]
          ids = params[:asset_id].split(',')
          update_kindeditor_assets_owner ids,@news.id,OwnerTypeHelper::NEWS
        end
        # ������ض�̬�ļ�¼add start
        # teachers = searchTeacherAndAssistant(@course)
        # for teacher in teachers
        #   if(teacher.user_id != User.current.id)
        #     notify = ActivityNotify.new()
        #     notify.activity_container_id = @course.id
        #     notify.activity_container_type = 'Course'
        #     notify.activity_id = @news.id
        #     notify.activity_type = 'News'
        #     notify.notify_to = teacher.user_id
        #     notify.is_read = 0
        #     notify.save()
        #   end
        # end
        # ������ض�̬�ļ�¼add end
        render_attachment_warning_if_needed(@news)
        flash[:notice] = l(:notice_successful_create)
        redirect_to course_news_index_url(@course)
      else
        layout_file =  'base_courses'
        render :action => 'new', :layout => layout_file
      end
    end
  end

  def edit
    if @course
      render :layout => "base_courses"
    end
  end

  def update
    @news.safe_attributes = params[:news]
    @news.save_attachments(params[:attachments])
    if @news.save
      render_attachment_warning_if_needed(@news)
      flash[:notice] = l(:notice_successful_update)
      redirect_to news_url(@news)
    else
      #flash[:error] = l(:notice_successful_update)
      redirect_to news_url(@news)
    end
  end

  def destroy
    @news.destroy
    # modify by nwb
    if @project
      redirect_to project_news_index_url(@project)
    elsif @course
      redirect_to course_news_index_url(@course)
    end

  end

  private

  def find_optional_project
    return true unless params[:project_id]
    @project = Project.find(params[:project_id])
    authorize
  rescue ActiveRecord::RecordNotFound
    render_404
  end
end
