# encoding: utf-8
#
# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

module MembersHelper
  def render_principals_for_new_members(project)
    scope = Principal.active.sorted.not_member_of(project).like(params[:q])
    principal_count = scope.count
    principal_pages = Redmine::Pagination::Paginator.new principal_count, 10, params['page'] #by young
    principals = scope.offset(principal_pages.offset).limit(principal_pages.per_page).all
    s = content_tag('div', principals_check_box_tags_ex('membership[user_ids][]', principals), :id => 'principals')
    links = pagination_links_full(principal_pages, principal_count, :per_page_links => false) {|text, parameters, options|
      link_to text, autocomplete_project_memberships_path(project, parameters.merge(:q => params[:q], :format => 'js')), :remote => true
    }
    s + content_tag('div', content_tag('ul', links), :class => 'pagination_new')
  end

  #获取项目可邀请的成员列表
  def render_project_members project
    if params[:q] && params[:q].lstrip.rstrip != ""
      scope = Principal.active.sorted.not_member_of(project).like(params[:q])
    else
      scope = []
    end
    principals = paginateHelper scope,10
    s = content_tag('ul', project_member_check_box_tags_ex('membership[user_ids][]', principals), :class => 'mb5', :id => 'principals')
    links = pagination_links_full(@obj_pages, @obj_count, :per_page_links => false, :remote => false, :flag => true){|text, parameters, options|
      link_to text, autocomplete_project_memberships_path(project, parameters.merge(:q => params[:q],:flag => true, :format => 'js')), :remote => true
    }
    s + content_tag('ul', links,:class => 'wlist', :id => "course_member_pagination_links" )
  end

  # add by nwb
  # 课程可添加的成员列表
  def render_principals_for_new_course_members(course)
    if params[:q] && params[:q].lstrip.rstrip != ""
      scope = Principal.active.sorted.not_member_of_course(course).like(params[:q])
    else
      scope = []
    end
    principals = paginateHelper scope,10
    s = content_tag('ul', project_member_check_box_tags_ex('membership[user_ids][]', principals), :class => 'mb5', :id => 'principals')

    links = pagination_links_full(@obj_pages, @obj_count, :per_page_links => false, :remote => false, :flag => true) {|text, parameters, options|
      link_to text, autocomplete_course_memberships_path(course, parameters.merge(:q => params[:q], :format => 'js')), :remote => true
    }

    s + content_tag('ul', links,:class => 'wlist',:id => "course_member_pagination_links")
  end

  # 新申请加入项目成员列表
  def render_principals_for_applied_members_new project
    scope = project.applied_projects.map(&:user)
    principals = paginateHelper scope,10
    s = content_tag('ul', principals_check_box_tags_li('membership[user_ids][]', principals), :class => 'mb5')
    links = pagination_links_full(@obj_pages, @obj_count, :per_page_links => false, :remote => false, :flag => true){|text, parameters, options|
      link_to text, appliedproject_project_memberships_path(project, parameters.merge(:q => params[:q],:flag => true, :format => 'js')), :remote => true
    }
    s + content_tag('ul', links,:class => 'wlist', :id => "course_member_pagination_links" )
  end

  # 当前申请加入的成员名单
  def render_principals_for_applied_members(project)
    scope = project.applied_projects.map(&:user)
    principal_count = scope.count
    principal_pages = Redmine::Pagination::Paginator.new principal_count, 10, params['page']
    offset ||= principal_pages.offset
    principals = scope[offset, 10]
    #principals = scope.offset(principal_pages.offset).limit(principal_pages.per_page).all
    #principals = ApplicationController.new.paginateHelper scope,10

    s = content_tag('div', principals_check_box_tags_ex('membership[user_ids][]', principals), :id => 'principals')

    links = pagination_links_full(principal_pages, principal_count, :per_page_links => false) {|text, parameters, options|
      link_to text, appliedproject_project_memberships_path(project, parameters.merge(:q => params[:q], :format => 'js')), :remote => true
    }

    s + content_tag('div', content_tag('ul', links), :class => 'applied_new')
  end

  private
  def paginateHelper obj, pre_size=20
    @obj_count = obj.count
    @obj_pages = Redmine::Pagination::Paginator.new @obj_count, pre_size, params['page']
    if obj.kind_of? ActiveRecord::Base or obj.kind_of?  ActiveRecord::Relation
      obj.limit(@obj_pages.per_page).offset(@obj_pages.offset)
    elsif obj.kind_of? Array
      obj[@obj_pages.offset, @obj_pages.per_page]
    else
      logger.error "[ApplicationController] Error : application_controller#paginateHelper ===> unknow category: #{obj.class}"
      raise RuntimeError, 'unknow type, Please input you type into this helper.'
    end
  end

end
