# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class Comment < ActiveRecord::Base
  include Redmine::SafeAttributes
  include ApplicationHelper
  has_many_kindeditor_assets :assets, :dependent => :destroy

  has_many :ActivityNotifies,:as => :activity, :dependent => :destroy
  # 课程/项目 消息
  has_many :course_messages, :class_name =>'CourseMessage', :as => :course_message, :dependent => :destroy
  has_many :forge_messages, :class_name => 'ForgeMessage', :as => :forge_message, :dependent => :destroy
  #end
  acts_as_event :datetime => :updated_on,
                :description => :comments,
                :type => 'news',
                :title=>Proc.new {|o| "RE: #{o.commented.title}" },
                :url => Proc.new {|o| {:controller => 'news', :action => 'show', :id => o.commented.id} }

  belongs_to :commented, :polymorphic => true, :counter_cache => true,:touch => true
  belongs_to :author, :class_name => 'User', :foreign_key => 'author_id'
  validates_presence_of :commented, :author, :comments
  safe_attributes 'comments'
  after_create :send_mail, :act_as_system_message, :act_as_student_score

  def act_as_system_message
    if self.commented.course
      if self.author_id != self.commented.author_id
        self.course_messages << CourseMessage.new(:user_id => self.commented.author_id, :course_id => self.commented.course.id, :viewed => false)
      end
    else  # 项目相关
      if self.author_id != self.commented.author_id
        self.forge_messages << ForgeMessage.new(:user_id => self.commented.author_id, :project_id => self.commented.project.id, :viewed => false)
      end
    end
  end

  def send_mail
    if self.commented.is_a?(News) && Setting.notified_events.include?('news_comment_added')
      Mailer.run.news_comment_added(self)
    end
  end
  after_destroy :delete_kindeditor_assets

  # Time 2015-03-31 09:15:06
  # Author lizanle
  # Description 删除对应评论的图片资源
  def delete_kindeditor_assets
    delete_kindeditor_assets_from_disk self.id,OwnerTypeHelper::COMMENT
  end

  def set_notify_id(notify_id)
    @notify_id= notify_id
  end

  def get_notify_id()
    return @notify_id
  end

  def set_notify_is_read(notify_is_read)
    @notify_is_read = notify_is_read
  end

  def get_notify_is_read()
    return @notify_is_read
  end

  # 课程成员得分（英雄榜）
  def act_as_student_score
    if self.commented.course
      unless self.author.allowed_to?(:as_teacher, self.commented.course)
        course_member_score(self.commented.course.id, self.author_id, "NewReply")
      end
    end
  end

end
