# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class Watcher < ActiveRecord::Base
  include UserScoreHelper
  belongs_to :watchable, :polymorphic => true
  belongs_to :user
  #Added by nie
  has_one :project_status
  has_one :users_status
  #end
  after_create :be_user_score
  after_destroy :down_user_score

  validates_presence_of :user
  validates_uniqueness_of :user_id, :scope => [:watchable_type, :watchable_id]
  validate :validate_user
  scope :visible, lambda {|*args|
    nil
  }
  
  # Unwatch things that users are no longer allowed to view
  def self.prune(options={})
    if options.has_key?(:user)
      prune_single_user(options[:user], options)
    else
      pruned = 0
      User.where("id IN (SELECT DISTINCT user_id FROM #{table_name})").all.each do |user|
        pruned += prune_single_user(user, options)
      end
      pruned
    end
  end

  protected

  def validate_user
    errors.add :user_id, :invalid unless user.nil? || user.active?
  end

  private

  def self.prune_single_user(user, options={})
    return unless user.is_a?(User)
    pruned = 0
    where(:user_id => user.id).all.each do |watcher|
      next if watcher.watchable.nil?

      if options.has_key?(:project)
        next unless watcher.watchable.respond_to?(:project) && watcher.watchable.project == options[:project]
      end

      if watcher.watchable.respond_to?(:visible?)
        unless watcher.watchable.visible?(user)
          watcher.destroy
          pruned += 1
        end
      end
    end
    pruned
  end

  # 更新用户分数     -by zjc
  def be_user_score
    #添加了关注
    if self.watchable_type == 'Principal'
      #影响力得分
      UserScore.influence(:followed_by, self.user,self.watchable,self, { watcher_id: self.id })
      update_follow(self.watchable,1)
    end

  end
 #减分操作    -by zjc
  def down_user_score
      #取消关注
    if self.watchable_type == 'Principal'
      UserScore.influence(:cancel_followed, self.user,self.watchable,self, { watcher_id: self.id })
      update_follow(self.watchable,1)
    end

  end
end
