require File.expand_path('../boot', __FILE__)

require 'rails/all'
require 'sprockets/railtie'
require 'elasticsearch/model'
if defined?(Bundler)
  # If you precompile assets before deploying to production, use this line
  Bundler.require(*Rails.groups(:assets => %w(development test)))
  # If you want your assets lazily compiled in production, use this line
  # Bundler.require(:default, :assets, Rails.env)
end

module RedmineApp
  class Application < Rails::Application
    # Settings in config/environments/* take precedence over those specified here.
    # Application configuration should go into files in config/initializers
    # -- all .rb files in that directory are automatically loaded.

    #verifier if email is real
   

    config.generators do |g|
         g.test_framework :rspec,
         fixtures: true,
         view_specs: false,
         helper_specs: false,
         routing_specs: false,
         controller_specs: true,
         request_specs: false
         g.fixture_replacement :factory_girl, dir: "spec/factories"
      end
    # Custom directories with classes and modules you want to be autoloadable.
    config.autoload_paths += %W(#{config.root}/lib)
    config.autoload_paths += %w(#{RAILS_ROOT}/app/sweepers)
    # Only load the plugins named here, in the order given (default is alphabetical).
    # :all can be used as a placeholder for all plugins not explicitly named.
    # config.plugins = [ :exception_notification, :ssl_requirement, :all ]

    # Activate observers that should always be running.
    config.active_record.observers = :journals_for_message_observer, :issue_observer, :journal_observer, :wiki_content_observer

    config.active_record.store_full_sti_class = true
    config.active_record.default_timezone = :local
    config.time_zone = 'Beijing'
    # Set Time.zone default to the specified zone and make Active Record auto-convert to this zone.
    # Run "rake -D time" for a list of tasks for finding time zone names. Default is UTC.
    # config.time_zone = 'Central Time (US & Canada)'

    # The default locale is :en and all translations from config/locales/*.rb,yml are auto loaded.
    # config.i18n.load_path += Dir[Rails.root.join('my', 'locales', '*.{rb,yml}').to_s]
    config.i18n.default_locale = :zh

    # Configure the default encoding used in templates for Ruby 1.9.
    config.encoding = "utf-8"

    # Configure sensitive parameters which will be filtered from the log file.
    config.filter_parameters += [:password]

    # Enable the asset pipeline
    config.assets.enabled = false

    # Version of your assets, change this if you want to expire all your assets
    config.assets.version = '1.0'

    config.action_mailer.perform_deliveries = false

    # Do not include all helpers
    config.action_controller.include_all_helpers = false

    config.action_view.sanitized_allowed_tags = 'div', 'p', 'span', 'img', 'embed'

    config.before_initialize do
    end

    config.after_initialize do
      if  RbConfig::CONFIG['target_os'] == 'mingw32'
        Elasticsearch::Client.new hosts: ['localhost:9200'], retry_on_failure: true,log:true
      elsif RbConfig::CONFIG['target_os'] == 'linux' &&  ["fast76"].include?(`hostname`.gsub("\n",""))
        Elasticsearch::Client.new hosts: ['localhost:9200'], retry_on_failure: true,log:true
      elsif RbConfig::CONFIG['target_os'] == 'linux' &&  ["testtrustie11","agent12"].include?(`hostname`.gsub("\n",""))
          Elasticsearch::Client.new hosts: ['localhost:9200','192.168.80.11:9200','192.168.80.12:9200'], retry_on_failure: true
      elsif  RbConfig::CONFIG['target_os'] == 'linux' &&  ["trustie168","trustieserver14","trustieserver16","Trustie18"].include?(`hostname`.gsub("\n",""))
         Elasticsearch::Client.new hosts: ['localhost:9200','192.168.80.168:9200'], retry_on_failure: true
      else
        Elasticsearch::Client.new hosts: ['localhost:9200'], retry_on_failure: true
      end
    end

    if File.exists?(File.join(File.dirname(__FILE__), 'additional_environment.rb'))
      instance_eval File.read(File.join(File.dirname(__FILE__), 'additional_environment.rb'))
    end

  end
end
