# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require 'redmine/core_ext'

begin
  require 'RMagick' unless Object.const_defined?(:Magick)
rescue LoadError
  # RMagick is not available
end

require 'redmine/scm/base'
require 'redmine/access_control'
require 'redmine/access_keys'
require 'redmine/activity'
require 'redmine/activity/fetcher'
require 'redmine/ciphering'
require 'redmine/codeset_util'
require 'redmine/custom_field_format'
require 'redmine/i18n'
require 'redmine/menu_manager'
require 'redmine/notifiable'
require 'redmine/platform'
require 'redmine/mime_type'
require 'redmine/notifiable'
require 'redmine/search'
require 'redmine/syntax_highlighting'
require 'redmine/thumbnail'
require 'redmine/unified_diff'
require 'redmine/utils'
require 'redmine/version'
require 'redmine/wiki_formatting'

require 'redmine/default_data/loader'
require 'redmine/helpers/calendar'
require 'redmine/helpers/diff'
require 'redmine/helpers/gantt'
require 'redmine/helpers/time_report'
require 'redmine/views/other_formats_builder'
require 'redmine/views/labelled_form_builder'
require 'redmine/views/builders'

require 'redmine/themes'
require 'redmine/hook'
require 'redmine/plugin'

if RUBY_VERSION < '1.9'
  require 'fastercsv'
else
  require 'csv'
  FCSV = CSV
end

Redmine::Scm::Base.add "Subversion"
Redmine::Scm::Base.add "Darcs"
Redmine::Scm::Base.add "Mercurial"
Redmine::Scm::Base.add "Cvs"
Redmine::Scm::Base.add "Bazaar"
Redmine::Scm::Base.add "Git"
Redmine::Scm::Base.add "Filesystem"

Redmine::CustomFieldFormat.map do |fields|
  fields.register 'string'
  fields.register 'text'
  fields.register 'int', :label => :label_integer
  fields.register 'float'
  fields.register 'list'
  fields.register 'date'
  fields.register 'bool', :label => :label_boolean
  fields.register 'user', :only => %w(Issue TimeEntry Version Project), :edit_as => 'list'
  fields.register 'version', :only => %w(Issue TimeEntry Version Project), :edit_as => 'list'
end

# Permissions
Redmine::AccessControl.map do |map|
  map.permission :view_project, {:projects => [:show], :activities => [:index]}, :public => true, :read => true
  map.permission :search_project, {:search => :index}, :public => true, :read => true
  map.permission :add_project, {:projects => [:new, :create]}, :require => :loggedin ,:belong_to_project => true
  map.permission :edit_project, {:projects => [:settings, :edit, :update]}, :require => :member ,:belong_to_project => true
  map.permission :close_project, {:projects => [:close, :reopen]}, :require => :member, :read => true ,:belong_to_project => true
  map.permission :select_project_modules, {:projects => :modules}, :require => :member ,:belong_to_project => true
  map.permission :select_contest_modules, {:contests => :modules}, :require => :member ,:belong_to_contest => true
  map.permission :manage_members, {:projects => :settings, :members => [:index, :show, :create, :update, :destroy, :autocomplete]}, :require => :member ,:belong_to_project => true
  map.permission :manage_versions, {:projects => :settings, :versions => [:new, :create, :edit, :update, :close_completed, :destroy]}, :require => :member ,:belong_to_project => true
  map.permission :add_subprojects, {:projects => [:new, :create]}, :require => :member ,:belong_to_project => true
  #错的权限，先注释掉
  #map.permission :view_journals_for_messages, {:gantts => [:show, :update]}, :read => true
  map.permission :upload_attachments,{:attachments => :upload},:require => :loggedin

  map.permission :quote_project, {},:require => :member,:belong_to_contest => true
  map.permission :is_manager,{},:require => :member ,:belong_to_project => true
  map.permission :as_teacher,{},:require => :member ,:belong_to_course => true
  map.permission :as_student,{},:require => :member ,:belong_to_course => true

  #课程权限模块
  map.permission :view_course, {:courses => [:show], :activities => [:index]}, :public => true, :read => true
  map.permission :search_course, {:search => :index}, :public => true, :read => true
  map.permission :add_course, {:courses => [:new, :create]}, :require => :loggedin ,:belong_to_course => true
  map.permission :edit_course, {:courses => [:settings, :edit, :update,:search_member]}, :require => :member  ,:belong_to_course => true
  map.permission :close_course, {:courses => [:close, :reopen]}, :require => :member, :read => true ,:belong_to_course => true
  map.permission :select_course_modules, {:courses => :modules}, :require => :member  ,:belong_to_course => true
  map.permission :view_course_journals_for_messages, {:courses => :feedback}, :require => :member,:read => true  ,:belong_to_course => true
  map.permission :memos_attachments_download,{:attachments => :download}
  map.permission :projects_attachments_download,{:attachments => :download},:belong_to_project => true
  map.permission :course_attachments_download,{:attachments => :download},:belong_to_course => true
  map.permission :contest_attachments_download,{:attachments => :download},:belong_to_contest => true
  map.permission :view_polls, {}, :public => true, :read => true
  map.permission :view_homeworks, {}, :public => true, :read => true

  map.course_module :files do |map|
  #  map.permission :manage_files, {:files => [:new, :create]}, :require => :loggedin
    map.permission :view_course_files, {:files => :index, :versions => :download}, :read => true,:belong_to_course => true
  end

    map.course_module :news do |map|
  #  map.permission :manage_news, {:news => [:new, :create, :edit, :update, :destroy], :comments => [:destroy]}, :require => :member
    map.permission :view_course_news, {:news => [:index, :show]}, :public => true, :read => true
  #  map.permission :comment_news, {:comments => :create}
    end

  map.course_module :bids do |map|
    map.permission :view_homework_attaches, {:bids => [:show, :show_project, :revision]}, :read => true,:belong_to_course => true
    map.permission :paret_in_homework,{},:require => :member ,:belong_to_course => true
  end

#  map.course_module :polls do |map|
#    map.permission :view_course_polls, {:polls => [:index,:show]},:require => :member, :belong_to_course => true
#  end

  map.course_module :boards do |map|
  #  map.permission :manage_boards, {:boards => [:new, :create, :edit, :update, :destroy]}, :require => :member
    map.permission :view_course_messages, {:boards => [:index, :show], :messages => [:show]}, :public => true, :read => true
  #  map.permission :add_messages, {:messages => [:new, :reply, :quote]}
  #  map.permission :edit_messages, {:messages => :edit}, :require => :member
  #  map.permission :edit_own_messages, {:messages => :edit}, :require => :loggedin
  #  map.permission :delete_messages, {:messages => :destroy}, :require => :member
  #  map.permission :delete_own_messages, {:messages => :destroy}, :require => :loggedin
  end

  #end

  map.project_module :issue_tracking do |map|
    # Issue categories
    map.permission :manage_categories, {:projects => :settings, :issue_categories => [:index, :show, :new, :create, :edit, :update, :destroy]}, :require => :member
    # Issues
    map.permission :view_issues, {:issues => [:index, :show],
                                  :auto_complete => [:issues],
                                  :context_menus => [:issues],
                                  :versions => [:index, :show, :status_by],
                                  :journals => [:index, :diff],
                                  :queries => :index,
                                  :reports => [:issue_report, :issue_report_details]},
                                  :read => true
    map.permission :add_issues, {:issues => [:new, :create, :update_form]}
    map.permission :edit_issues, {:issues => [:edit, :update, :bulk_edit, :bulk_update, :update_form], :journals => [:new]}
    map.permission :manage_issue_relations, {:issue_relations => [:index, :show, :create, :destroy]}
    map.permission :manage_subtasks, {}
    map.permission :set_issues_private, {}
    map.permission :set_own_issues_private, {}, :require => :loggedin
    map.permission :add_issue_notes, {:issues => [:edit, :update, :update_form], :journals => [:new]}
    map.permission :edit_issue_notes, {:journals => :edit}, :require => :loggedin
    map.permission :edit_own_issue_notes, {:journals => :edit}, :require => :loggedin
    map.permission :view_private_notes, {}, :read => true, :require => :member
    map.permission :set_notes_private, {}, :require => :member
    map.permission :move_issues, {:issues => [:bulk_edit, :bulk_update]}, :require => :loggedin
    map.permission :delete_issues, {:issues => :destroy}, :require => :member
    # Queries
    map.permission :manage_public_queries, {:queries => [:new, :create, :edit, :update, :destroy]}, :require => :member
    map.permission :save_queries, {:queries => [:new, :create, :edit, :update, :destroy]}, :require => :loggedin
    # Watchers
    map.permission :view_issue_watchers, {}, :read => true
    map.permission :add_issue_watchers, {:watchers => [:new, :create, :append, :autocomplete_for_user]}
    map.permission :delete_issue_watchers, {:watchers => :destroy}
  end

  map.project_module :time_tracking do |map|
    map.permission :log_time, {:timelog => [:new, :create]}, :require => :loggedin
    map.permission :view_time_entries, {:timelog => [:index, :report, :show]}, :read => true
    map.permission :edit_time_entries, {:timelog => [:edit, :update, :destroy, :bulk_edit, :bulk_update]}, :require => :member
    map.permission :edit_own_time_entries, {:timelog => [:edit, :update, :destroy,:bulk_edit, :bulk_update]}, :require => :loggedin
    map.permission :manage_project_activities, {:project_enumerations => [:update, :destroy]}, :require => :member
  end

  map.project_module :news do |map|
    map.permission :manage_news, {:news => [:new, :create, :edit, :update, :destroy], :comments => [:destroy]}, :require => :member
    map.permission :view_news, {:news => [:index, :show]}, :public => true, :read => true
    map.permission :comment_news, {:comments => :create}
  end

  map.contest_module :contestnotifications do |map|
    map.permission :manage_contestnotifications, {:contestnotifications => [:new, :create, :edit, :update, :destroy], :notificationcomments => [:destroy]}, :require => :member,:belong_to_contest => true
    map.permission :view_contestnotifications, {:contestnotifications => [:index, :show]}, :public => true, :read => true
    map.permission :notificationcomment_contestnotifications, {:notificationcomments => :create},:belong_to_contest => true
  end

  map.project_module :documents do |map|
    map.permission :add_documents, {:documents => [:new, :create, :add_attachment]}, :require => :loggedin
    map.permission :edit_documents, {:documents => [:edit, :update, :add_attachment]}, :require => :loggedin
    map.permission :delete_documents, {:documents => [:destroy]}, :require => :loggedin
    map.permission :view_documents, {:documents => [:index, :show, :download]}, :read => true
  end

  map.project_module :files do |map|
    map.permission :manage_files, {:files => [:new, :create]}, :require => :loggedin
    map.permission :view_files, {:files => :index, :versions => :download}, :read => true
  end

  map.project_module :wiki do |map|
    map.permission :manage_wiki, {:wikis => [:edit, :destroy]}, :require => :member
    map.permission :rename_wiki_pages, {:wiki => :rename}, :require => :member
    map.permission :delete_wiki_pages, {:wiki => [:destroy, :destroy_version]}, :require => :member
    map.permission :view_wiki_pages, {:wiki => [:index, :show, :special, :date_index]}, :read => true
    map.permission :export_wiki_pages, {:wiki => [:export]}, :read => true
    map.permission :view_wiki_edits, {:wiki => [:history, :diff, :annotate]}, :read => true
    map.permission :edit_wiki_pages, :wiki => [:edit, :update, :preview, :add_attachment]
    map.permission :delete_wiki_pages_attachments, {}
    map.permission :protect_wiki_pages, {:wiki => :protect}, :require => :member
  end

  map.project_module :repository do |map|
    map.permission :manage_repository, {:repositories => [:new, :create, :edit, :update, :committers, :destroy]}, :require => :member
    map.permission :browse_repository, {:repositories => [:show, :browse, :entry, :raw, :annotate, :changes, :diff, :stats, :graph]}, :read => true
    map.permission :view_changesets, {:repositories => [:show, :revisions, :revision]}, :read => true
    map.permission :commit_access, {}
    map.permission :manage_related_issues, {:repositories => [:add_related_issue, :remove_related_issue]}
  end

  map.project_module :boards do |map|
    map.permission :manage_boards, {:boards => [:new, :create, :edit, :update, :destroy]}, :require => :member
    map.permission :view_messages, {:boards => [:index, :show], :messages => [:show]}, :public => true, :read => true
    map.permission :add_messages, {:messages => [:new, :reply, :quote]}
    map.permission :edit_messages, {:messages => :edit}, :require => :member
    map.permission :edit_own_messages, {:messages => :edit}, :require => :loggedin
    map.permission :delete_messages, {:messages => :destroy}, :require => :member
    map.permission :delete_own_messages, {:messages => :destroy}, :require => :loggedin
  end

  map.project_module :calendar do |map|
    map.permission :view_calendar, {:calendars => [:show, :update]}, :read => true
  end

  map.project_module :gantt do |map|
    map.permission :view_gantt, {:gantts => [:show, :update]}, :read => true
  end

  map.project_module :dts do |map|
    map.permission :do_dts, {:dts => :show}, :read => true
  end
  # map.project_module :journals do |map|
  #   map.permission :view_journals_for_messages, {:gantts => [:show, :update]}, :read => true
  # end

end
#by young
Redmine::MenuManager.map :top_menu do |menu|
  menu.push :home, {:host => Setting.host_user}
  menu.push :course_practice, {:controller => 'projects', :action => 'course', :project_type => 1}
  menu.push :project_deposit, { :controller => 'projects', :action => 'index', :project_type => 0}, :caption => :label_project_deposit
  menu.push :software_user, {:controller => 'users', :action => 'index'}
  menu.push :contest_innovate, {:controller => 'bids', :action => 'contest', :project_type => 1}
  menu.push :requirement_enterprise, {:controller => 'bids', :action => 'index'}
  menu.push :project_module_forums, :forums_path
  menu.push :stores_index, :stores_path
  
 
  # menu.push :investor, :home_path
  # menu.push :theme, :home_path
  # menu.push :my_page, { :controller => 'my', :action => 'page' }, :if => Proc.new { User.current.logged? }
  # menu.push :administration, { :controller => 'admin', :action => 'index' }, :if => Proc.new { User.current.admin? }, :last => true
  # menu.push :help, Redmine::Info.help_url, :last => true
end
#end

# by huang
# Redmine::MenuManager.map :top_home_menu do |menu|
  # menu.push :home, :home_path
  # menu.push :projects, { :controller => 'projects', :action => 'index' }, :caption => :label_project_plural
  # menu.push :developer, {:controller => 'users', :action => 'index'}
  # menu.push :investor, :home_path
  # menu.push :theme, :home_path
  # menu.push :my_page, { :controller => 'my', :action => 'page' }, :if => Proc.new { User.current.logged? }
  # menu.push :administration, { :controller => 'admin', :action => 'index' }, :if => Proc.new { User.current.admin? }, :last => true
  # menu.push :help, Redmine::Info.help_url, :last => true
# end

# Redmine::MenuManager.map :top_enterprise_menu do |menu| 
  # menu.push :home, :home_path
  # menu.push :requirement, {:controller => 'bids', :action => 'index', :project_type => 0 }
  # menu.push :projects, { :controller => 'projects', :action => 'index', :project_type => 0 }, :caption => :label_project_plural
#   
  # menu.push :developer, {:controller => 'users', :action => 'index', :project_type => 0 }
  # menu.push :college_into, {:controller=>'projects', :action=>'course_enterprise', :project_type => 1 }
  # # menu.push :investor, :home_path
  # # menu.push :theme, :home_path
  # # menu.push :my_page, { :controller => 'my', :action => 'page' }, :if => Proc.new { User.current.logged? }
  # # menu.push :administration, { :controller => 'admin', :action => 'index' }, :if => Proc.new { User.current.admin? }, :last => true
  # # menu.push :help, Redmine::Info.help_url, :last => true
# end

# Redmine::MenuManager.map :top_college_menu do |menu|
  # menu.push :home, :home_path
  # menu.push :new_course, {:controller => 'projects', :action => 'course', :project_type => 1}
  # menu.push :new_homework, {:controller => 'bids', :action => 'index', :project_type => 1 }
  # menu.push :new_contest, {:controller => 'bids', :action => 'contest', :project_type => 1}
  # menu.push :projects, { :controller => 'projects', :action => 'index', :project_type => 0 }, :caption => :label_project_plural  
  # menu.push :developer, {:controller => 'users', :action => 'index', :project_type => 1 }
  # menu.push :enterprise_into, {:controller=>'projects', :action=>'enterprise_course', :project_type => 0 }
  # menu.push :investor, :home_path
  # menu.push :theme, :home_path
  # menu.push :my_page, { :controller => 'my', :action => 'page' }, :if => Proc.new { User.current.logged? }
  # menu.push :administration, { :controller => 'admin', :action => 'index' }, :if => Proc.new { User.current.admin? }, :last => true
  # menu.push :help, Redmine::Info.help_url, :last => true
# end
# end

Redmine::MenuManager.map :account_menu do |menu|
  menu.push :login, {:controller => 'account', :action => 'login'}, :if => Proc.new { !User.current.logged? }
  menu.push :register, {:controller => 'account', :action => 'register'}, :if => Proc.new { !User.current.logged? && Setting.self_registration? }
  # menu.push :my_account, { :controller => 'my', :action => 'account' }, :if => Proc.new { User.current.logged? }
  menu.push :logout, {:controller => 'account', :action => 'logout'}, :html => {:method => 'post'}, :if => Proc.new { User.current.logged? }
end
########fq
Redmine::MenuManager.map :bid_menu do |menu|
  menu.push :respond, { :controller => 'bids', :action => 'show' }, :caption => :label_user_response
  menu.push :project, { :controller => 'bids', :action => 'show_project', :host => Setting.host_name }, :caption => :label_bidding_project
  # menu.push :result,  { :controller => 'bids', :action => 'show_results' }, 
             # :caption => :label_bidding_results,:if => Proc.new{ |p| User.current.id == p } 
end
###new add by linchun 
Redmine::MenuManager.map :contest_menu do |menu|
  menu.push :respond, :show_contest_contest_path, :caption => :label_user_response
  #menu.push :project, :show_project_contest_path, :caption => :label_contest_project
  #menu.push :application, :show_softapplication_contest_path, :caption => :label_contest_application
  menu.push :attendingcontest, {:controller => 'contests', :action => 'show_attendingcontest'}, :caption => :label_attending_contest
  menu.push :contestnotifications, { :controller => 'contestnotifications', :action => 'index' }, :param => :contest_id, :caption => :label_contest_notification
  #menu.push :contestnotification, {:controller => 'contests', :action => 'show_notification'}, :caption => :label_contest_notification
  # menu.push :attendingcontest, :show_attendingcontest_contest_path, :caption => :label_attendin,g_contest
  # menu.push :result,  { :controller => 'bids', :action => 'show_results' }, 
             # :caption => :label_bidding_results,:if => Proc.new{ |p| User.current.id == p } 
end

####
Redmine::MenuManager.map :application_menu do |menu|
  # Empty
end

######
Redmine::MenuManager.map :homework_menu do |menu|
  menu.push :respond, { :controller => 'bids', :action => 'show' }, :caption => :label_homework_response
  menu.push :project, { :controller => 'bids', :action => 'show_project' }, :caption => :label_bidding_homework
end
########end
Redmine::MenuManager.map :admin_menu do |menu|
  menu.push :organization, {:controller => 'admin', :action => 'organization'}, :caption => :label_organization_list
  menu.push :projects, {:controller => 'admin', :action => 'projects'}, :caption => :label_project_plural
  menu.push :courses, {:controller => 'admin', :action => 'courses'}, :caption => :label_course_all
  menu.push :users, {:controller => 'admin', :action => 'users'}, :caption => :label_user_plural
  menu.push :messages, {:controller => 'admin', :action => 'messages'}, :caption => :label_system_message
  menu.push :schools, {:controller => 'admin', :action => 'schools'}, :caption => :label_school_plural
  menu.push :first_page_made, {:controller => 'admin',:action => 'first_page_made'},:caption => :label_first_page_made
  menu.push :mobile_version, {:controller => 'admin',:action => 'mobile_version'},:caption => :label_mobile_version
  menu.push :groups, {:controller => 'groups'}, :caption => :label_group_plural
  menu.push :roles, {:controller => 'roles'}, :caption => :label_role_and_permissions
  menu.push :trackers, {:controller => 'trackers'}, :caption => :label_tracker_plural
  menu.push :issue_statuses, {:controller => 'issue_statuses'}, :caption => :label_issue_status_plural,
            :html => {:class => 'issue_statuses'}
  menu.push :workflows, {:controller => 'workflows', :action => 'edit'}, :caption => :label_workflow
  menu.push :custom_fields, {:controller => 'custom_fields'},  :caption => :label_custom_field_plural,
            :html => {:class => 'custom_fields'}
  menu.push :enumerations, {:controller => 'enumerations'}
  menu.push :settings, {:controller => 'settings'}
  menu.push :ldap_authentication, {:controller => 'auth_sources', :action => 'index'},
            :html => {:class => 'server_authentication'}
  menu.push :plugins, {:controller => 'admin', :action => 'plugins'}, :last => true
  menu.push :info, {:controller => 'admin', :action => 'info'}, :caption => :label_information_plural, :last => true
  menu.push :leave_messages, {:controller => 'admin', :action => 'leave_messages'}, :caption => :label_leave_message_list
  menu.push :messages_list, {:controller => 'admin', :action => 'messages_list'}, :caption => :label_message_plural
  menu.push :notices, {:controller => 'admin', :action => 'notices'}, :caption => :label_notification_list
  menu.push :latest_login_users, {:controller => 'admin', :action => 'latest_login_users'}, :caption => :label_latest_login_user_list
  menu.push :homework, {:controller => 'admin', :action => 'homework'}, :caption => :label_user_homework

end
#Modified by young
Redmine::MenuManager.map :project_menu do |menu|
  menu.push :overview, { :controller => 'projects', :action => 'show' }
#  menu.push :activity, { :controller => 'activities', :action => 'index' }
  #menu.push :roadmap, { :controller => 'versions', :action => 'index' }, :param => :project_id
#              :if => Proc.new { |p| p.shared_versions.any? }
  menu.push :issues, { :controller => 'issues', :action => 'index' },:if => Proc.new {|p| p.enabled_module_names.include?('issue_tracking') } ,:param => :project_id, :caption => :label_issue_plural
#  menu.push :new_issue, { :controller => 'issues', :action => 'new', :copy_from => nil }, :param => :project_id, :caption => :label_issue_new,
#              :html => { :accesskey => Redmine::AccessKeys.key_for(:new_issue) }
#  menu.push :gantt, { :controller => 'gantts', :action => 'show' }, :param => :project_id, :caption => :label_gantt
#  menu.push :calendar, { :controller => 'calendars', :action => 'show' }, :param => :project_id, :caption => :label_calendar
  menu.push :news, { :controller => 'news', :action => 'index' }, :if => Proc.new {|p| p.enabled_module_names.include?('news') },:param => :project_id, :caption => :label_news_plural
#  menu.push :documents, { :controller => 'documents', :action => 'index' }, :param => :project_id, :caption => :label_document_plural
#  menu.push :wiki, { :controller => 'wiki', :action => 'show', :id => nil }, :param => :project_id,
#              :if => Proc.new { |p| p.wiki && !p.wiki.new_record? }
  menu.push :boards, { :controller => 'boards', :action => 'index', :id => nil }, :param => :project_id,
              :if => Proc.new { |p| p.boards.any? && p.enabled_module_names.include?('boards') }, :caption => :label_board_plural
  #menu.push :files, { :controller => 'files', :action => 'index' }, :param => :project_id, :caption => :label_file_new
  menu.push :repository, { :controller => 'repositories', :action => 'show', :repository_id => nil, :path => nil, :rev => nil },
              :if => Proc.new { |p| p.repository && !p.repository.new_record? && !( !User.current.member_of?(p) && p.hidden_repo ) && p.enabled_module_names.include?('repository') }
  menu.push :settings, { :controller => 'projects', :action => 'settings' }, :last => true
  
end
#end

#added by young
Redmine::MenuManager.map :course_menu do |menu|
  menu.push :overview, { :controller => 'projects', :action => 'show'}
  menu.push :homework, { :controller => 'projects', :action => 'homework' }
#  menu.push :news, { :controller => 'news', :action => 'index' }, :param => :project_id, :caption => :label_news_plural
#  menu.push :files, { :controller => 'files', :action => 'index' }, :caption => :label_file_new, :param => :project_id
  menu.push :repository, { :controller => 'repositories', :action => 'show', :repository_id => nil, :path => nil, :rev => nil, :course => 1 }     
 # menu.push :settings, { :controller => 'projects', :action => 'settings', :course => 1 }, :last => true
  
end
Redmine::MenuManager.map :user_menu do |menu|
  menu.push :activity, {:controller => 'users', :action => 'show', :host => Setting.host_user }
  menu.push :user_course, {:controller => 'users', :action => 'user_courses'},
            :if => Proc.new {|u| if User.current == u || User.current.admin?
                                    membership = u.coursememberships.all
                                  else
                                    membership = u.coursememberships.all(:conditions => Course.visible_condition(User.current))
                                  end
                                  FirstPage.find_by_page_type('project').show_course != 2 && membership && membership.count > 0
                             }
  #menu.push :user_homework, {:controller => 'users', :action => 'user_homeworks'} by huang
  menu.push :user_project, {:controller => 'users', :action => 'user_projects_index', :host => Setting.host_name},
            :if => Proc.new {|u| if User.current.admin?
                                   memberships = u.memberships.all(conditions: "projects.project_type = #{Project::ProjectType_project}").first
                                 else
                                   cond = Project.visible_condition(User.current) + " AND projects.project_type <> 1"
                                   memberships = u.memberships.all(:conditions => cond).first
                                 end
                                 watch_projects = Project.joins(:watchers).where("project_type <>? and watchable_type = ? and `watchers`.user_id = ?", '1','Project', u.id).first
                                 memberships || watch_projects
            }
  # menu.push :requirement_focus, {:controller => 'users', :action => 'watch_bids'} by huang
  menu.push :user_newfeedback, {:controller => 'users', :action => 'user_newfeedback', :host => Setting.host_user}

end 
Redmine::MenuManager.map :user_enterprise_menu do |menu|
  menu.push :activity, {:controller => 'users', :action => 'show', :host => Setting.host_user }
 
  menu.push :user_project, {:controller => 'users', :action => 'user_projects', :host => Setting.host_name}
  
  # menu.push :requirement_focus, {:controller => 'users', :action => 'watch_bids', :host => Setting.host_user}
  menu.push :user_newfeedback, {:controller => 'users', :action => 'user_newfeedback', :host => Setting.host_user}
  
end
Redmine::MenuManager.map :user_menu_self do |menu|
  menu.push :activity, {:controller => 'users', :action => 'show', :host => Setting.host_name }
  menu.push :user_information, {:controller => 'users', :action => 'info', :host => Setting.host_user}  
  menu.push :user_project, {:controller => 'users', :action => 'user_projects', :host => Setting.host_name}
  # menu.push :requirement_focus, {:controller => 'users', :action => 'watch_bids', :host => Setting.host_user}
  menu.push :user_newfeedback, {:controller => 'users', :action => 'user_newfeedback', :host => Setting.host_user}
  
end
#end

Redmine::Activity.map do |activity|
  activity.register :issues, :class_name => %w(Issue Journal)
  activity.register :changesets
  activity.register :news
  activity.register :contestnotification
  activity.register :documents, :class_name => %w(Document Attachment)
  activity.register :files, :class_name => 'Attachment'
  activity.register :course_files, :class_name => 'Attachment'
  activity.register :wiki_edits, :class_name => 'WikiContent::Version', :default => false
  activity.register :messages, :default => false
  activity.register :time_entries, :default => false
  # added by fq
  activity.register :bids, :class_name => 'Bid'
  activity.register :memos, :class_name => 'Memo'
  activity.register :journals_for_messages
  # end
  #added by nwb
  activity.register :course_journals_for_messages , :class_name => 'JournalsForMessage'
  activity.register :course_news, :class_name => 'News'
  activity.register :course_messages, :default => false, :class_name => 'Message'

  activity.register :homeworks, :class_name => 'Bid'
  activity.register :polls, :class_name => 'Poll'
end

Redmine::Search.map do |search|
  search.register :issues
  search.register :news
  search.register :contestnotification
  search.register :documents
  search.register :changesets
  search.register :wiki_pages
  search.register :messages
  search.register :projects
end

Redmine::WikiFormatting.map do |format|
  format.register :textile, Redmine::WikiFormatting::Textile::Formatter, Redmine::WikiFormatting::Textile::Helper
end

ActionView::Template.register_template_handler :rsb, Redmine::Views::ApiTemplateHandler
